@@ -23,7 +23,6 @@
     #include <aescpp.h>
 #endif //MEDIAINFO_AES
 #include "MediaInfo/HashWrapper.h"
-#include <cstring>
 //---------------------------------------------------------------------------
 
 namespace MediaInfoLib
@@ -31,6 +30,39 @@ namespace MediaInfoLib
 
 class MediaInfo_Internal;
 
+template <class T> inline Ztring Get_Hex_ID(const T& Value)
+{
+    Ztring ID_String;
+    ID_String.From_Number(Value); 
+    ID_String += __T(" (0x"); 
+    ID_String += Ztring::ToZtring(Value, 16); 
+    ID_String += __T(")");
+    return ID_String;
+}
+
+struct buffer_data
+{
+    size_t Size;
+    int8u* Data;
+
+    buffer_data()
+    {
+        Size = 0;
+        Data = NULL;
+    }
+    buffer_data(const int8u* aData, size_t aSize)
+    {
+        Size = aSize;
+        Data = new int8u[aSize];
+        std::memcpy(Data, aData, aSize);
+    }
+
+    ~buffer_data()
+    {
+        delete[] Data; //Data=NULL;
+    }
+};
+
 #if !MEDIAINFO_TRACE
     #include "MediaInfo/File__Analyze_MinimizeSize.h"
 #else
@@ -57,12 +89,14 @@ public :
     void    Open_Buffer_Init        (File__Analyze* Sub);
     void    Open_Buffer_Init        (File__Analyze* Sub, int64u File_Size);
     void    Open_Buffer_OutOfBand   (                    const int8u* Buffer, size_t Buffer_Size) {File__Analyze::Buffer=Buffer; File__Analyze::Buffer_Size=Buffer_Size; Element_Offset=0; Element_Size=Buffer_Size; Read_Buffer_OutOfBand(); File__Analyze::Buffer=NULL; File__Analyze::Buffer_Size=0; Element_Offset=0; Element_Size=0;}
-    void    Open_Buffer_OutOfBand   (File__Analyze* Sub);
+    void    Open_Buffer_OutOfBand   (File__Analyze* Sub                     , size_t Buffer_Size);
+    void    Open_Buffer_OutOfBand   (File__Analyze* Sub) {Open_Buffer_OutOfBand(Sub, Element_Size-Element_Offset);};
     void    Open_Buffer_Continue    (                    const int8u* Buffer, size_t Buffer_Size);
     void    Open_Buffer_Continue    (File__Analyze* Sub, const int8u* Buffer, size_t Buffer_Size, bool IsNewPacket=true, float64 Ratio=1.0);
     void    Open_Buffer_Continue    (File__Analyze* Sub, size_t Buffer_Size) {if (Element_Offset+Buffer_Size<=Element_Size) Open_Buffer_Continue(Sub, Buffer+Buffer_Offset+(size_t)Element_Offset, Buffer_Size); Element_Offset+=Buffer_Size;}
     void    Open_Buffer_Continue    (File__Analyze* Sub) {if (Element_Offset<=Element_Size) Open_Buffer_Continue(Sub, Buffer+Buffer_Offset+(size_t)Element_Offset, (size_t)(Element_Size-Element_Offset)); Element_Offset=Element_Size;}
     void    Open_Buffer_Position_Set(int64u File_Offset);
+    void    Open_Buffer_CheckFileModifications();
     #if MEDIAINFO_SEEK
     size_t  Open_Buffer_Seek        (size_t Method, int64u Value, int64u ID);
     #endif //MEDIAINFO_SEEK
@@ -235,6 +269,7 @@ protected :
     virtual void Read_Buffer_Init ()          {}; //Temp, should be in File__Base caller
     virtual void Read_Buffer_OutOfBand ()     {Open_Buffer_Continue(Buffer, Buffer_Size);} //Temp, should be in File__Base caller
     virtual void Read_Buffer_Continue ()      {}; //Temp, should be in File__Base caller
+    virtual void Read_Buffer_CheckFileModifications() {} //Temp, should be in File__Base caller
     virtual void Read_Buffer_AfterParsing ()  {}; //Temp, should be in File__Base caller
     #if MEDIAINFO_SEEK
     virtual size_t Read_Buffer_Seek (size_t, int64u, int64u); //Temp, should be in File__Base caller
@@ -359,7 +394,7 @@ protected :
         if (Config_Trace_Level<=0.7)
             return;
 
-        if (Parameter && std::string(Parameter) == "NOK")
+        if ((Parameter && std::string(Parameter) == "NOK") || (Measure && std::string(Measure) == "Error"))
             Element[Element_Level].TraceNode.HasError = true;
 
         Element[Element_Level].TraceNode.Infos.push_back(new element_details::Element_Node_Info(Parameter, Measure, AfterComma));
@@ -376,6 +411,8 @@ protected :
     inline void Element_Info_From_Milliseconds (int64u Parameter)                  {if (Config_Trace_Level<1) return; Element_Info(Ztring().Duration_From_Milliseconds(Parameter));}
 
     void Element_Parser (const char* Name);
+    void Element_Error (const char* Name);
+    void Param_Error (const char* Name);
 
     //Elements - End
     inline void Element_End () {Element_End_Common_Flush();}
@@ -407,8 +444,6 @@ public :
         else
             return Ztring().From_Local(Value, Value_Size);
     }
-    #define VALUE(Value) \
-        Ztring::ToZtring(Value).MakeUpperCase()+__T(" (0x")+Ztring::ToZtring(Value, 16).MakeUpperCase()+__T(")")
 
     //Param - Main
     template<typename T>
@@ -485,7 +520,7 @@ public :
         if (Config_Trace_Level<=0.7)
             return;
 
-        if (Parameter && std::string(Parameter) == "NOK")
+        if ((Parameter && std::string(Parameter) == "NOK") || (Measure && std::string(Measure) == "Error"))
             Element[Element_Level].TraceNode.HasError = true;
 
         int32s child = Element[Element_Level].TraceNode.Current_Child;
@@ -1405,6 +1440,9 @@ public :
     #else //MEDIAINFO_DEMUX
         #define Demux(_A, _B, _C)
     #endif //MEDIAINFO_DEMUX
+    #if MEDIAINFO_DECODE
+        void Decoded(const int8u* Buffer, size_t Buffer_Size);
+    #endif //MEDIAINFO_DECODE
 
     //Events data
     bool    PES_FirstByte_IsAvailable;
